
#include "nanova.h"
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>

#define NB_VALUES 19
#define NB_LEVELS 3
#define NB_GROUPS 7
#define NB_TESTS 4

int test_una(long int num_test, unsigned int nb_levels, 
	     unsigned int nb_groups, unsigned int nb_values, 
	     unsigned int *tree, unsigned int *groups, double *values,
	     struct una_results *results)
{
	int res, i, j;

	printf("*********************************\n"
	       "Test %li\n\n", num_test);

	res=unbalanced_nested_anova(nb_levels, nb_groups, nb_values,
				     tree, groups, values, results);

	if (res!=0) {
		printf("Argh: erreur %i\n", res);
#define TREE(i,j) tree[i*nb_levels+j]
		printf("Tree:\n");
		for(i=0; i<nb_groups; i++) {
			printf("group %i [%i]: ", i, groups[i]);
			for(j=0; j<nb_levels; j++) {
				printf("%d, ", TREE(i,j));
			}
			printf("\n");
		}
		return 1;
	}

	for(i=0; i<nb_levels; i++) {
		printf("Level %i: p_value=%f, F=%f\n", i, results[i].p_value,
		       results[i].F);
	}
	printf("\n");
	return 0;
}

unsigned int tree0[NB_LEVELS*NB_GROUPS]={
	0, 0, 0,
	0, 0, 1,
	0, 1, 2,
	1, 2, 3,
	1, 2, 4, 
	1, 3, 5,
	1, 3, 6,
};
unsigned int tree1[NB_LEVELS*NB_GROUPS]={
	0, 0, 0,
	0, 0, 1,
	0, 1, 2,
	1, 2, 3,
	1, 2, 7, 
	1, 3, 5,
	1, 3, 6,
};
unsigned int tree2[NB_LEVELS*NB_GROUPS]={
	0, 0, 0,
	0, 0, 1,
	0, 1, 2,
	1, 2, 3,
	1, 2, 4, 
	1, 4, 5,
	1, 4, 6,
};
unsigned int tree3[NB_LEVELS*NB_GROUPS]={
	0, 0, 0,
	0, 0, 1,
	0, 1, 2,
	1, 2, 3,
	2, 2, 4, 
	2, 3, 5,
	2, 3, 6,
};
unsigned int* trees[NB_TESTS]={
	tree0, tree1, tree2, tree3
};

int main(int argc, char **argv) {

	double values[NB_VALUES]={0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
				  10, 11, 12, 13, 14, 15, 16, 17, 18};
	struct una_results results[NB_LEVELS];
	unsigned int groups[NB_GROUPS]={1, 3, 4, 3, 2, 2, 4};

	long int num_test;
	char *endptr;
	
	/* VINCE Comment j'initialise results? 
	   Plutot la ou plutot dans la fonction au dessus? */
	int i;
	
	for (i=0; i<NB_LEVELS; i++) {
		results[i].p_value=0;
		results[i].F=0;
	}
	
	if (argc==2) {
		num_test=strtol(argv[1], &endptr, 0);
		if (*endptr=='\0' //&& errno==0
			&& num_test>=0 && num_test<NB_TESTS) {
			test_una(num_test, NB_LEVELS, NB_GROUPS, NB_VALUES,
				 trees[num_test], groups, values, results);
		} else {
			if (errno) {
				perror("strtol");
			}
			fprintf(stderr, "Bad parameter %i, %i\n", errno, num_test);
			return 1;
		}
	} else if (argc<2) {
		int i;
		for (i=0; i<NB_TESTS; i++) {
			test_una(100+i, NB_LEVELS, NB_GROUPS, NB_VALUES,
				 trees[i], groups, values, results);
		}
	} else {		
		fprintf(stderr, "Too many parameters");
		return 1;
	}

	return 0;
}
